'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var useAsyncReducer = _interopDefault(require('use-async-reducer'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function getUpdaterPromise(asyncUpdater) {
  if (typeof asyncUpdater === 'function') {
    return asyncUpdater();
  }

  return asyncUpdater;
}

function useAsyncCall(asyncCreator, options = {}) {
  const _useAsyncReducer = useAsyncReducer(options.initialValue),
        _useAsyncReducer2 = _slicedToArray(_useAsyncReducer, 2),
        response = _useAsyncReducer2[0],
        actions = _useAsyncReducer2[1];

  react.useEffect(() => {
    let didCancel = false;

    const callAsync =
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(function* () {
        actions.request();

        try {
          const data = yield asyncCreator();

          if (didCancel) {
            return;
          }

          actions.success(data);

          if (typeof options.onSuccess === 'function') {
            options.onSuccess(data);
          }
        } catch (error) {
          if (didCancel) {
            return;
          }

          actions.failure(error);

          if (typeof options.onFailure === 'function') {
            options.onFailure(error);
          }
        } finally {
          if (didCancel) {
            return;
          }

          if (typeof options.onComplete === 'function') {
            options.onComplete();
          }
        }
      });

      return function callAsync() {
        return _ref.apply(this, arguments);
      };
    }();

    callAsync();
    return () => {
      didCancel = true;
    };
  }, [asyncCreator, options.onSuccess, options.onFailure, options.onComplete]);
  const update = react.useCallback(
  /*#__PURE__*/
  function () {
    var _ref2 = _asyncToGenerator(function* (asyncUpdater, updateOptions = {}) {
      actions.request();

      try {
        const data = yield getUpdaterPromise(asyncUpdater);

        actions.success(data);

        if (typeof updateOptions.onSuccess === 'function') {
          updateOptions.onSuccess(data);
        }

        return data;
      } catch (error) {

        if (updateOptions.saveError) {
          actions.failure(error);
        } else {
          actions.complete();
        }

        if (typeof updateOptions.onFailure === 'function') {
          updateOptions.onFailure(error);
        }

        if (updateOptions.throwError) {
          throw error;
        }
      } finally {

        if (typeof updateOptions.onComplete === 'function') {
          updateOptions.onComplete();
        }
      }
    });

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }(), [actions]);
  return [response, update];
}

exports.default = useAsyncCall;
